--[[
AssignedTankAssist by Akryn

TODO:
Changelog:

1.1.6 (9-5-2012):
-Updated for 5.0.4

1.1.5 (12-10-2011):
-Updated TOC for 4.3

1.1.4 (10-19-2010):
-Update for WoW 4.0 (thanks to acidzero for providing the fix, since I don't have an account ATM).

1.1.2 (2-9-2010):
-!mt and !ma tags work properly now (note that the tags are designed for raids that only use 1, tags like this are pretty useless if you use multiple mt's or ma's since there's no way to differentiate between them).

1.1.1 (2-8-2010):
-The !mt and !ma tags work better now.
-Fix for "raidx" spam and lag during raid updates (sorry x.x).

1.1 (2-5-2010):
-Added !mt and !ma for raid "maintank" and "mainassist" roles.
-Fix for settings getting cleared when changing other addons' settings without opening ATA's config panel.
-Fixed a few typos.

1.0 (1-27-2010):
-You can now have up to 30 macros.
-Fixed the "default" button not resetting all the options.
-Added "Clear Binding" button.

0.1 B10:
-Added an option to suppress error text.
-Added an option to run the macro when called even if there is not a tank (replaces !t with blank space).
-Added a button to bind a key to the macro rather than using the /click command.

0.1 B9:
-Redesign, see the info page.

0.1 B8:
-Check for changes occasionally as well as on events. 

0.1 B7:
-Bug fix.

0.1 B6:
-Added some customization stuff to the top of this file.

0.1 B5:
-Fix for rarely leaving duplicate bindings.
-Allows using a macro (ATA_Temp) instead of a binding (see description).
-Registers PLAYER_ROLES_ASSIGNED in addition to PARTY_MEMBERS_CHANGED

0.1 B4:
-Bug fixes

0.1 B3:
-Bug fixes

0.1 B2:
-Bug fixes

]]

AssignedTankAssist_Vars = {
	macroText = "/assist !t"	
}

local L = {
	["Optional keybinding:"]						= "Binding:",
	["Silence warnings:"]							= "No warnings:",
	["Run even if no tank:"]						= "Run even if no tank:",
	["Please press the key you would like to use."]				= "Please press the key you would like to use.",
	["/run print(\"There is no macro text assigned yet.\")"] 		= "/run print(\"There is no macro text assigned yet.\")",
	["/run print(\"There is no macro text assigned.\")"]			= "/run print(\"There is no macro text assigned.\")",
	["/run print(\"Nothing to do because there is no tank.\")"]		= "/run print(\"Nothing to do because there is no tank.\")",
	["/run print(\"Nothing to do because you aren't in a LFD group.\")"]	= "/run print(\"Nothing to do because you aren't in a group.\")",
	["<None>"]								= "<None>",
	["Clear Binding"]							= "Clear Binding",
	["Enter a macro in the larger input box below, use !t anywhere the tank's name should go.\n\nCall this from a macro or keybind with:"] = "Enter a macro in the larger input box below. Use !t anywhere the tank's name should go.\n\nYou can call this from a macro with:",
}

local numPages = 30

local me = CreateFrame("BUTTON", "AssignedTankAssistButton", UIParent, "SecureActionButtonTemplate")
me:SetAttribute("type", "macro")
me:SetAttribute("macro", false)
me:SetAttribute("macrotext", L["/run print(\"There is no macro text assigned yet.\")"])
me:RegisterForClicks("AnyDown")

for i = 2, numPages do
	local me = CreateFrame("BUTTON", "AssignedTankAssistButton" .. i, UIParent, "SecureActionButtonTemplate")
	me.number = i
	me:SetAttribute("type", "macro")
	me:SetAttribute("macro", false)
	me:SetAttribute("macrotext", L["/run print(\"There is no macro text assigned yet.\")"])
	me:RegisterForClicks("AnyDown")
end

local bindPromptFrame = CreateFrame("frame")
bindPromptFrame:SetParent("UIParent")
bindPromptFrame:SetAllPoints()
bindPromptFrame:SetBackdrop({
	bgFile = "Interface/Tooltips/UI-Tooltip-Background",
	edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
	tile = true, tileSize = 16, edgeSize = 16,
	insets = {left = 4, right = 4, top = 4, bottom = 4}
})
bindPromptFrame:SetBackdropColor(0, 0, 0, .6)
bindPromptFrame:Hide()

local function ClearBindings(...)
	for i = 1, select("#", ...) do
		local b = select(i, ...)
		if b then
			SetBinding(b)
		end
	end	
end

local function changeBinding(button, num)
	if num == 1 then num = nil end
	local buttonName = "AssignedTankAssistButton" .. (num or "")

	local badKeys = {
		BUTTON1 = true,
		BUTTON2 = true,
		LSHIFT = true,
		RSHIFT = true,
		LCTRL = true,
		RCTRL = true,
		LALT = true,
		RALT = true,
		UNKNOWN = true
	}

	if type(button) == "string" then
		if button:find("Button") then
			if button == "LeftButton" then
				button = "BUTTON1"
			elseif button == "RightButton" then
				button = "BUTTON2"
			elseif button == "MiddleButton" then
				button = "BUTTON3"
			else
				button = button:upper()
			end
		end		
		
		if not badKeys[button] then
			if IsShiftKeyDown() then
				button = "SHIFT-"..button
			end
			if IsControlKeyDown() then
				button = "CTRL-"..button
			end
			if IsAltKeyDown() then
				button = "ALT-"..button
			end

			ClearBindings(GetBindingKey("CLICK " .. buttonName .. ":LeftButton"))
			SetBindingClick(button, buttonName)
		else
			return
		end
	else
		ClearBindings(GetBindingKey("CLICK " .. buttonName .. ":LeftButton"))
	end
	
	bindPromptFrame:Hide()
	AssignedTankAssist_setBindButton:Hide()
	AssignedTankAssist_setBindButton:Show()
end

bindPromptFrame:SetFrameStrata("DIALOG")
bindPromptFrame:EnableMouse(true)
bindPromptFrame:EnableKeyboard(true)

local bindString = bindPromptFrame:CreateFontString()
bindString:SetFontObject("GameFontNormal")
bindString:ClearAllPoints()
bindString:SetPoint("CENTER", bindPromptFrame, "CENTER")
bindString:SetText(L["Please press the key you would like to use."])

local function SetMacro(raid)
	if not AssignedTankAssist_Vars.otherMacroTexts then
		AssignedTankAssist_Vars.otherMacroTexts = {}
	end

	if InCombatLockdown() then
		me:RegisterEvent("PLAYER_REGEN_ENABLED")
		return
	end
	
	me:UnregisterEvent("PLAYER_REGEN_ENABLED")
	--logic hacked to make it do multiple buttons, make it better at some point...
	local curButton = me
	local mt, ma = "nil", "nil"
	local UE = UnitExists
	
	for i = 1, numPages do
		if raid then
			if GetNumGroupMembers() < 2 then return end			
			
			for ri = 1, 40 do
				if UE("raid" .. ri) then
					if GetPartyAssignment("MAINTANK", "raid" .. ri) then
						mt = ri
					end
					if GetPartyAssignment("MAINASSIST", "raid" .. ri) then
						ma = ri
					end
					if ma and mt then
						ri = 41
					end
				end
			end
			if (ma ~= "nil") or (mt ~= "nil") then			
				local man = UnitName("raid" .. ma) or ""
				local mtn = UnitName("raid" .. mt) or ""
				local macroText
				if i > 1 then
					macroText = AssignedTankAssist_Vars.otherMacroTexts[i]
				else
					macroText = AssignedTankAssist_Vars.macroText
				end
				if not macroText then
					macroText = (AssignedTankAssist_Vars.noErrorText and "" or L["/run print(\"There is no macro text assigned.\")"])
				end
				macroText = macroText:gsub("!t", "")
				macroText = macroText:gsub("!ma", man)
				macroText = macroText:gsub("!mt", mtn)
				if mtn ~= "" then
					macroText = macroText:gsub("!tt", mtn)
				else
					macroText = macroText:gsub("!tt", man)
				end
				curButton:SetAttribute("macrotext", macroText)
			else
				if AssignedTankAssist_Vars.alwaysRun then
					local macroText
					if i > 1 then
						macroText = AssignedTankAssist_Vars.otherMacroTexts[i]
					else
						macroText = AssignedTankAssist_Vars.macroText
					end
					if not macroText then
						macroText = (AssignedTankAssist_Vars.noErrorText and "" or L["/run print(\"There is no macro text assigned.\")"])
					end
					macroText = macroText:gsub("!t", "")
					macroText = macroText:gsub("!ma", "")
					macroText = macroText:gsub("!mt", "")
					macroText = macroText:gsub("!tt", "")
					curButton:SetAttribute("macrotext", macroText)
				else
					curButton:SetAttribute("macrotext",  (AssignedTankAssist_Vars.noErrorText and "" or L["/run print(\"Nothing to do because there is no tank.\")"]) )
				end
			end
		else --not raid
			--if GetNumGroupMembers() > 2 then return end
			if GetNumGroupMembers() > 1 then
				local u, p
				local success = false
				for u = 1, 4 do
					p = "party" .. u

					if UnitGroupRolesAssigned(p) == "TANK" then
						success = true
						local r = UnitName(p) or ""
						local macroText
						if i > 1 then
							macroText = AssignedTankAssist_Vars.otherMacroTexts[i]
						else
							macroText = AssignedTankAssist_Vars.macroText
						end
						if not macroText then
							macroText = (AssignedTankAssist_Vars.noErrorText and "" or L["/run print(\"There is no macro text assigned.\")"])
						end
						macroText = macroText:gsub("!t", r)
						macroText = macroText:gsub("!ma", "")
						macroText = macroText:gsub("!mt", r)
						macroText = macroText:gsub("!tt", r)
						curButton:SetAttribute("macrotext", macroText)
					end
				end
				if not success then
					if AssignedTankAssist_Vars.alwaysRun then
						local macroText
						if i > 1 then
							macroText = AssignedTankAssist_Vars.otherMacroTexts[i]
						else
							macroText = AssignedTankAssist_Vars.macroText
						end
						if not macroText then
							macroText = (AssignedTankAssist_Vars.noErrorText and "" or L["/run print(\"There is no macro text assigned.\")"])
						end
						macroText = macroText:gsub("!t", "")
						macroText = macroText:gsub("!ma", "")
						macroText = macroText:gsub("!mt", "")
						macroText = macroText:gsub("!tt", "")
						curButton:SetAttribute("macrotext", macroText)
					else
						curButton:SetAttribute("macrotext",  (AssignedTankAssist_Vars.noErrorText and "" or L["/run print(\"Nothing to do because there is no tank.\")"]) )
					end
				end
			else
				if AssignedTankAssist_Vars.alwaysRun then
					local macroText
					if i > 1 then
						macroText = AssignedTankAssist_Vars.otherMacroTexts[i]
					else
						macroText = AssignedTankAssist_Vars.macroText
					end
					if not macroText then
						macroText = (AssignedTankAssist_Vars.noErrorText and "" or L["/run print(\"There is no macro text assigned.\")"])
					end
					macroText = macroText:gsub("!t", "")
					macroText = macroText:gsub("!ma", "")
					macroText = macroText:gsub("!mt", "")
					macroText = macroText:gsub("!tt", "")
					curButton:SetAttribute("macrotext", macroText)
				else
					curButton:SetAttribute("macrotext",  (AssignedTankAssist_Vars.noErrorText and "" or L["/run print(\"Nothing to do because you aren't in a LFD group.\")"]) )
				end
			end
		end
				
		curButton = _G["AssignedTankAssistButton" .. (i + 1)]
	end
end

me:RegisterEvent("PLAYER_ROLES_ASSIGNED")
me:RegisterEvent("GROUP_ROSTER_UPDATE")
me:RegisterEvent("VARIABLES_LOADED")

me:SetScript("OnEvent", function(self, event) SetMacro((event == "GROUP_ROSTER_UPDATE")) end)

me.config = CreateFrame("frame")
me.config.name = "Assigned Tank Assist"
me.config.pane = 1
me.config:Hide()
local t = me.config:CreateFontString()
t:ClearAllPoints()
t:SetPoint("TOPLEFT", me.config, "TOPLEFT", 10, -10)
t:SetWidth(400)
t:SetFontObject(GameFontNormal)
t:SetText(L["Enter a macro in the larger input box below, use !t anywhere the tank's name should go.\n\nCall this from a macro or keybind with:"])
t:SetJustifyH("LEFT")

bindPromptFrame:SetScript("OnMouseDown", function(self, button) changeBinding(button, me.config.pane) end)
bindPromptFrame:SetScript("OnKeyDown", function(self, button) changeBinding(button, me.config.pane) end)

me.config.inputBox = CreateFrame("EditBox", "AssignedTankAssist_InputBox", me.config, "InputBoxTemplate")
me.config.copyBox = CreateFrame("EditBox", "AssignedTankAssist_CopyBox", me.config, "InputBoxTemplate")
me.config.alwaysRun = CreateFrame("CheckButton", "AssignedTankAssist_alwaysRun", me.config, "OptionsCheckButtonTemplate")
me.config.noErrorText = CreateFrame("CheckButton", "AssignedTankAssist_noErrorText", me.config, "OptionsCheckButtonTemplate")
me.config.setBindButton = CreateFrame("Button", "AssignedTankAssist_setBindButton", me.config, "GameMenuButtonTemplate")
me.config.clearBinding = CreateFrame("Button", "AssignedTankAssist_clearBinding", me.config, "GameMenuButtonTemplate")
me.config.paneButtons = {}
for i = 1, numPages do
	me.config.paneButtons[i] = CreateFrame("Button", "AssignedTankAssist_paneButton" .. i, me.config, "GameMenuButtonTemplate")
end

AssignedTankAssist_InputBoxLeft:SetTexture(nil)
AssignedTankAssist_InputBoxRight:SetTexture(nil)
AssignedTankAssist_InputBoxMiddle:SetTexture(nil)

me.config.inputBox:ClearFocus()
me.config.inputBox:SetAutoFocus(false)
me.config.inputBox:SetMultiLine(true)
me.config.inputBox:ClearAllPoints()
me.config.inputBox:SetWidth(300)
me.config.inputBox:SetPoint("TOPLEFT", me.config, "TOPLEFT", 10, -100)
me.config.inputBox:SetPoint("BOTTOMLEFT", me.config, "TOPLEFT", 10, -380)
me.config.inputBox:SetBackdrop({bgFile = [[Interface\ChatFrame\ChatFrameBackground]], tile = false, edgeSize = 0})
me.config.inputBox:SetBackdropColor(0, 0, 0, 1)
me.config.inputBox:SetScript("OnShow", function(self)
	local showText
	if me.config.pane > 1 then
		showText = AssignedTankAssist_Vars.otherMacroTexts[me.config.pane]
	else
		showText = AssignedTankAssist_Vars.macroText
	end
	showText = showText or ""
	self:SetText(showText)
end)

me.config.copyBox:SetAutoFocus(false)
me.config.copyBox:SetMultiLine(false)
me.config.copyBox:ClearAllPoints()
me.config.copyBox:SetPoint("TOPLEFT", me.config, "TOPLEFT", 10, -70)
me.config.copyBox:SetWidth(200)
me.config.copyBox:SetHeight(20)
me.config.copyBox:SetParent(me.config)
local function setTheText(self)
	self:SetText("/click AssignedTankAssistButton" .. (me.config.pane > 1 and me.config.pane or ""))
	self:SetScript("OnTextChanged", setTheText)
	_G["AssignedTankAssist_InputBoxLeft"]:SetTexture(nil)
	_G["AssignedTankAssist_InputBoxRight"]:SetTexture(nil)
	_G["AssignedTankAssist_InputBoxMiddle"]:SetTexture(nil)
end
me.config.copyBox:SetScript("OnShow", setTheText)

me.config.noErrorText:ClearAllPoints()
me.config.noErrorText:SetPoint("TOPLEFT", me.config.inputBox, "BOTTOMLEFT", 50, -20)
me.config.noErrorText:SetHitRectInsets(-10,-10,-10,-10)
me.config.noErrorText:SetScript("OnShow", function(self) self:SetChecked(AssignedTankAssist_Vars.noErrorText) end)
local noErrorTextLabel = AssignedTankAssist_noErrorTextText
noErrorTextLabel:ClearAllPoints()
noErrorTextLabel:SetPoint("BOTTOMRIGHT", me.config.noErrorText, "TOPRIGHT", 0, 3)
noErrorTextLabel:SetText(L["Silence warnings:"])

me.config.alwaysRun:ClearAllPoints()
me.config.alwaysRun:SetPoint("TOPLEFT", me.config.noErrorText, "TOPRIGHT", 100, 0)
me.config.alwaysRun:SetHitRectInsets(-10,-10,-10,-10)
me.config.alwaysRun:SetScript("OnShow", function(self) self:SetChecked(AssignedTankAssist_Vars.alwaysRun) end)
local alwaysRunLabel = AssignedTankAssist_alwaysRunText
alwaysRunLabel:ClearAllPoints()
alwaysRunLabel:SetPoint("BOTTOMRIGHT", me.config.alwaysRun, "TOPRIGHT", 1, 3)
alwaysRunLabel:SetText(L["Run even if no tank:"])

me.config.setBindButton:SetWidth(180)
me.config.setBindButton:SetHeight(20)
me.config.setBindButton:ClearAllPoints()
me.config.setBindButton:SetPoint("TOPLEFT", me.config.alwaysRun, "TOPRIGHT", 10, 0)
me.config.setBindButton:SetScript("OnShow", function(self) _G[self:GetName() .. "Text"]:SetText((GetBindingKey("CLICK AssignedTankAssistButton" .. (me.config.pane > 1 and me.config.pane or "") .. ":LeftButton")) or L["<None>"]) end)
me.config.setBindButton:SetScript("OnClick", function(self) bindPromptFrame:Show() end)
local setBindLabel = me.config.setBindButton:CreateFontString()
setBindLabel:SetFontObject("GameFontNormal")
setBindLabel:ClearAllPoints()
setBindLabel:SetPoint("BOTTOMLEFT", me.config.setBindButton, "TOPLEFT", 0, 3)
setBindLabel:SetText(L["Optional keybinding:"])

me.config.clearBinding:SetWidth(100)
me.config.clearBinding:SetHeight(15)
me.config.clearBinding:ClearAllPoints()
me.config.clearBinding:SetPoint("BOTTOMRIGHT", me.config.setBindButton, "TOPRIGHT", -4, -1)
me.config.clearBinding:SetScript("OnShow", function(self) _G[self:GetName() .. "Text"]:SetText(L["Clear Binding"]) end)
me.config.clearBinding:SetScript("OnClick", function() ClearBindings(GetBindingKey("CLICK AssignedTankAssistButton" .. (me.config.pane > 1 and me.config.pane or "") .. ":LeftButton")) AssignedTankAssist_setBindButton:Hide() AssignedTankAssist_setBindButton:Show() end)

local function pageTo(num)
	if me.config.pane > 1 then
		AssignedTankAssist_Vars.otherMacroTexts[me.config.pane] = me.config.inputBox:GetText()
	else
		AssignedTankAssist_Vars.macroText = me.config.inputBox:GetText()
	end
	
	me.config.paneButtons[me.config.pane]:Enable()
	me.config.paneButtons[num]:Disable()
	me.config.pane = num
	me.config.inputBox:Hide()
	me.config.inputBox:Show()
	me.config.setBindButton:Hide()
	me.config.setBindButton:Show()
	setTheText(me.config.copyBox)
end

for i = 1, numPages do
	local c = me.config.paneButtons[i]
	c.num = i
	c:SetWidth(30)
	c:SetHeight(15)
	c:ClearAllPoints()
	local halfWay = numPages / 2
	c:SetPoint("TOPLEFT", me.config.inputBox, "TOPRIGHT", 15 + ((i > halfWay) and 45 or 0), 0 -(((i - ((i > halfWay) and halfWay or 0) - 1) * 15)) + 1)
	c:SetScript("OnShow", function(self) _G[self:GetName() .. "Text"]:SetText(i) end)
	c:SetScript("OnClick", function(self) pageTo(self.num) end)
end

local optionsBackgroundFrame = CreateFrame("frame")
optionsBackgroundFrame:SetParent(me.config)
optionsBackgroundFrame:ClearAllPoints()
optionsBackgroundFrame:SetPoint("TOPLEFT", noErrorTextLabel, "TOPLEFT", -3, 5)
optionsBackgroundFrame:SetPoint("BOTTOMRIGHT", me.config.setBindButton, "BOTTOMRIGHT", 9, -5)
optionsBackgroundFrame:SetBackdrop({
	bgFile = "Interface/Tooltips/UI-Tooltip-Background",
	edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
	tile = true, tileSize = 16, edgeSize = 16,
	insets = {left = 4, right = 4, top = 4, bottom = 4}
})
optionsBackgroundFrame:SetBackdropColor(0, 0, 0, .9)
optionsBackgroundFrame:SetFrameLevel(me.config.noErrorText:GetFrameLevel() - 1)

me.config.okay = function()
	if not me.config:IsVisible() then return end
	me.config.saving = true
	if me.config.pane > 1 then
		AssignedTankAssist_Vars.otherMacroTexts[me.config.pane] = me.config.inputBox:GetText()
	else
		AssignedTankAssist_Vars.macroText = me.config.inputBox:GetText()
	end
	AssignedTankAssist_Vars.alwaysRun = me.config.alwaysRun:GetChecked()
	AssignedTankAssist_Vars.noErrorText = me.config.noErrorText:GetChecked()
	SetMacro(IsInRaid())
	SaveBindings(GetCurrentBindingSet())
end

me.config.default = function()
	if not me.config:IsVisible() then return end
	pageTo(1)
	AssignedTankAssist_Vars.otherMacroTexts = {}
	me.config.inputBox:SetText("/assist !t")
	me.config.noErrorText:SetChecked(false)
	me.config.alwaysRun:SetChecked(false)
	for i = 1, numPages do
		changeBinding(nil, i)
	end
end

me.config.saving = true
me.config.cancel = function()
	if not me.config:IsVisible() then return end
	if me.config.saving then return end
	AssignedTankAssist_Vars.macroText = me.config.macroTextOnOpen
	AssignedTankAssist_Vars.otherMacroTexts = me.config.macroTextsOnOpen
	LoadBindings(GetCurrentBindingSet())
end

me.config:SetScript("OnShow", function()
	me.config.saving = false
	AssignedTankAssist_Vars.otherMacroTexts = AssignedTankAssist_Vars.otherMacroTexts or {}
	me.config.paneButtons[me.config.pane]:Disable()
	me.config.macroTextOnOpen = AssignedTankAssist_Vars.macroText
	me.config.macroTextsOnOpen = AssignedTankAssist_Vars.otherMacroTexts and CopyTable(AssignedTankAssist_Vars.otherMacroTexts) or {}
end)
me.config:SetScript("OnHide", me.config.cancel)

InterfaceOptions_AddCategory(me.config)

SlashCmdList["ASSIGNEDTANKASSIST"] = function() InterfaceOptionsFrame_OpenToCategory(me.config) end
SLASH_ASSIGNEDTANKASSIST1 = "/atassist"

